@@ -8,7 +8,7 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = '0.0.26'
+__version__ = '0.0.31'
 
 import sys
 
@@ -27,6 +27,7 @@ setup = wandb_sdk.setup
 save = wandb_sdk.save
 watch = wandb_sdk.watch
 login = wandb_sdk.login
+Artifact = wandb_sdk.Artifact
 Settings = wandb_sdk.Settings
 
 from wandb.apis import InternalApi, PublicApi
@@ -58,6 +59,7 @@ from wandb import superagent
 from wandb.viz import visualize
 from wandb import plots
 
+
 # Used to make sure we don't use some code in the incorrect process context
 _IS_INTERNAL_PROCESS = False
 
@@ -79,6 +81,7 @@ from wandb.lib.ipython import _get_python_type
 
 # globals
 Api = PublicApi
+api = InternalApi()
 run = None
 config = _preinit.PreInitObject("wandb.config")
 summary = _preinit.PreInitObject("wandb.summary")
@@ -87,6 +90,12 @@ join = _preinit.PreInitCallable("wandb.join")
 
 keras = _lazyloader.LazyLoader('wandb.keras', globals(), 'wandb.framework.keras')
 sklearn = _lazyloader.LazyLoader('wandb.sklearn', globals(), 'wandb.sklearn')
+tensorflow = _lazyloader.LazyLoader('wandb.tensorflow', globals(), 'wandb.tensorflow')
+xgboost = _lazyloader.LazyLoader('wandb.xgboost', globals(), 'wandb.framework.xgboost')
+
+def ensure_configured():
+    global api
+    api = InternalApi()
 
 __all__ = [
     "__version__",
